@@ -278,7 +278,7 @@ ptrheap_delete(struct ptrheap * H, size_t rc)
 
 /**
  * ptrheap_deletemin(H):
- * Delete the minimum element in the heap ${H}.
+ * Delete the minimum element in the heap ${H}.  The heap must not be empty.
  */
 void
 ptrheap_deletemin(struct ptrheap * H)
@@ -288,6 +288,20 @@ ptrheap_deletemin(struct ptrheap * H)
 	ptrheap_delete(H, 0);
 }
 
+/**
+ * ptrheap_decrease(H, rc):
+ * Adjust the heap ${H} to account for the fact that the element ptr for
+ * which the function call setreccookie(cookie, ptr, ${rc}) was most recently
+ * made has decreased.
+ */
+void
+ptrheap_decrease(struct ptrheap * H, size_t rc)
+{
+
+	/* Move the element up if necessary. */
+	heapifyup(H->elems, rc, H->compar, H->setreccookie, H->cookie);
+}
+
 /**
  * ptrheap_increase(H, rc):
  * Adjust the heap ${H} to account for the fact that the element ptr for
