@@ -1,5 +1,6 @@
 #include <sys/time.h>
 
+#include <signal.h>
 #include <stdlib.h>
 #include <string.h>
 
@@ -19,6 +20,9 @@ MPOOL(eventrec, struct eventrec, 4096);
 /* Zero timeval, for use with non-blocking event runs. */
 static const struct timeval tv_zero = {0, 0};
 
+/* We want to interrupt a running event loop. */
+static volatile sig_atomic_t interrupt_requested = 0;
+
 /**
  * events_mkrec(func, cookie):
  * Package ${func}, ${cookie} into a struct eventrec.
@@ -79,10 +83,13 @@ doevent(struct eventrec * r)
  * associated with expired timers registered via events_timer_register will
  * be run.  If any event function returns a non-zero result, no further
  * events will be run and said non-zero result will be returned; on error,
- * -1 will be returned.
+ * -1 will be returned.  May be interrupted by events_interrupt, in which case
+ * 0 will be returned.  If there are runnable events, events_run is guaranteed
+ * to run at least one; but it may return while there are still more runnable
+ * events.
  */
-int
-events_run(void)
+static int
+_events_run(void)
 {
 	struct eventrec * r;
 	struct timeval * tv;
@@ -96,6 +103,10 @@ events_run(void)
 			if ((rc = doevent(r)) != 0)
 				goto done;
 
+			/* Interrupt loop if requested. */
+			if (interrupt_requested)
+				goto done;
+
 			/* Get the next event. */
 			r = events_immediate_get();
 		}
@@ -110,16 +121,20 @@ events_run(void)
 	 */
 	if (events_timer_min(&tv))
 		goto err0;
-	if (events_network_select(tv))
+	if (events_network_select(tv, &interrupt_requested))
 		goto err1;
 	free(tv);
 
 	/*
 	 * Check for available immediate events, network events, and timer
 	 * events, in that order of priority; exit only when no more events
-	 * are available.
+	 * are available or when interrupted.
 	 */
 	do {
+		/* Interrupt loop if requested. */
+		if (interrupt_requested)
+			goto done;
+
 		/* Run an immediate event, if one is available. */
 		if ((r = events_immediate_get()) != NULL) {
 			if ((rc = doevent(r)) != 0)
@@ -136,7 +151,7 @@ events_run(void)
 
 		/* Check if any new network events are available. */
 		memcpy(&tv2, &tv_zero, sizeof(struct timeval));
-		if (events_network_select(&tv2))
+		if (events_network_select(&tv2, &interrupt_requested))
 			goto err0;
 		if ((r = events_network_get()) != NULL) {
 			if ((rc = doevent(r)) != 0)
@@ -168,11 +183,28 @@ err0:
 	return (-1);
 }
 
+/* Wrapper function for events_run to reset interrupt_requested. */
+int
+events_run(void)
+{
+	int rc;
+
+	/* Call the real function. */
+	rc = _events_run();
+
+	/* Reset interrupt_requested after quitting the loop. */
+	interrupt_requested = 0;
+
+	/* Return status. */
+	return (rc);
+}
+
 /**
  * events_spin(done):
- * Run events until ${done} is non-zero (and return 0), an error occurs (and
+ * Call events_run until ${done} is non-zero (and return 0), an error occurs (and
  * return -1), or a callback returns a non-zero status (and return the status
- * code from the callback).
+ * code from the callback).  May be interrupted by events_interrupt (and return
+ * 0).
  */
 int
 events_spin(int * done)
@@ -180,24 +212,36 @@ events_spin(int * done)
 	int rc = 0;
 
 	/* Loop until we're done or have a non-zero status. */
-	while ((done[0] == 0) && (rc == 0)) {
+	while ((done[0] == 0) && (rc == 0) && (interrupt_requested == 0)) {
 		/* Run events. */
-		rc = events_run();
+		rc = _events_run();
 	}
 
+	/* Reset interrupt_requested after quitting the loop. */
+	interrupt_requested = 0;
+
 	/* Return status code. */
 	return (rc);
 }
 
+/**
+ * events_interrupt(void):
+ * Halt the event loop after finishing the current event.  This function can
+ * be safely called from within a signal handler.
+ */
+void
+events_interrupt(void)
+{
+
+	/* Interrupt the event loop. */
+	interrupt_requested = 1;
+}
+
 /**
  * events_shutdown(void):
- * Clean up and free memory.  This call is not necessary on program exit and
- * is only expected to be useful when checking for memory leaks.
+ * Deprecated function; does nothing.
  */
 void
 events_shutdown(void)
 {
-
-	events_network_shutdown();
-	events_timer_shutdown();
 }
