@@ -19,6 +19,29 @@ struct push {
 	int out;
 };
 
+static void
+workthread_cleanup(void * cookie)
+{
+	struct push * P = cookie;
+
+	/* Close the descriptor (we either hit EOF, or received a cancel). */
+	close(P->in);
+
+	/*
+	 * Try to shut down the descriptor we're writing to.  Ignore ENOTSOCK,
+	 * since it might, indeed, not be a socket.
+	 */
+	if (shutdown(P->out, SHUT_WR)) {
+		if (errno != ENOTSOCK) {
+			warnp("Error shutting down socket");
+			exit(1);
+		}
+	}
+
+	/* Free our parameters. */
+	free(P);
+}
+
 /* Bit-pushing thread. */
 static void *
 workthread(void * cookie)
@@ -26,6 +49,9 @@ workthread(void * cookie)
 	struct push * P = cookie;
 	ssize_t readlen;
 
+	/* Set up cleanup function. */
+	pthread_cleanup_push(workthread_cleanup, P);
+
 	/* Infinite loop unless we hit EOF or an error. */
 	do {
 		/* Read data and die on error. */
@@ -48,36 +74,22 @@ workthread(void * cookie)
 		}
 	} while (1);
 
-	/* Close the descriptor we hit EOF on. */
-	close(P->in);
-
-	/*
-	 * Try to shut down the descriptor we're writing to.  Ignore ENOTSOCK,
-	 * since it might, indeed, not be a socket.
-	 */
-	if (shutdown(P->out, SHUT_WR)) {
-		if (errno != ENOTSOCK) {
-			warnp("Error shutting down socket");
-			exit(1);
-		}
-	}
-
-	/* Free our parameters. */
-	free(P);
+	/* Clean up. */
+	pthread_cleanup_pop(1);
 
 	/* We're done. */
 	return (NULL);
 }
 
 /**
- * pushbits(in, out):
- * Create a thread which copies data from ${in} to ${out}.
+ * pushbits(in, out, thr):
+ * Create a thread which copies data from ${in} to ${out} and
+ * store the thread ID in ${thr}.
  */
 int
-pushbits(int in, int out)
+pushbits(int in, int out, pthread_t * thr)
 {
 	struct push * P;
-	pthread_t thr;
 	int rc;
 
 	/* Allocate structure. */
@@ -87,7 +99,7 @@ pushbits(int in, int out)
 	P->out = out;
 
 	/* Create thread. */
-	if ((rc = pthread_create(&thr, NULL, workthread, P)) != 0) {
+	if ((rc = pthread_create(thr, NULL, workthread, P)) != 0) {
 		warn0("pthread_create: %s", strerror(rc));
 		goto err1;
 	}
