@@ -7,6 +7,8 @@
 #   gave it -o 1).
 
 ### Constants
+c_valgrind_min=1
+flag_1="${s_basename}-1.flag"
 
 
 ### Actual command
@@ -19,10 +21,11 @@ scenario_cmd() {
 	setup_check_variables
 	(
 		( echo "" ; sleep 2 ) |		\
-			 ${nc_client_binary} [127.0.0.1]:${src_port}
+			 ${nc_client_binary} ${src_sock}
 		echo $? > ${c_exitfile}
+		touch "${flag_1}"
 	) &
-	sleep 3
+	wait_for_file "${flag_1}"
 
 	# Wait for server(s) to quit.
 	servers_stop
