@@ -8,6 +8,7 @@
 # - the received file should match lorem-send.txt
 
 ### Constants
+c_valgrind_min=1
 ncat_output="${s_basename}-ncat-output.txt"
 
 ### Actual command
@@ -28,7 +29,7 @@ scenario_cmd() {
 	setup_check_variables
 	(
 		cat ${scriptdir}/lorem-send.txt |	\
-			${nc_client_binary} [127.0.0.1]:${src_port}
+			${nc_client_binary} ${src_sock}
 		echo $? > ${c_exitfile}
 	)
 
